@@ -279,7 +279,6 @@ PIXCMAP   *cmap;
 }
 
 
-
 /*!
  *  pixColorGrayCmap()
  *
@@ -295,18 +294,17 @@ PIXCMAP   *cmap;
  *          preserving antialiasing.
  *          If type == L_PAINT_DARK, it colorizes non-white pixels,
  *          preserving antialiasing.
- *      (3) If box is NULL, applies function to the entire image; otherwise,
- *          clips the operation to the intersection of the box and pix.
- *      (4) This can also be called through pixColorGray().
- *      (5) This increases the colormap size by the number of
+ *      (3) box gives the region to apply color; if NULL, this
+ *          colorizes the entire image.
+ *      (4) If the cmap is only 2 or 4 bpp, pixs is converted in-place
+ *          to an 8 bpp cmap.  A 1 bpp cmap is not a valid input pix.
+ *      (5) This can also be called through pixColorGray().
+ *      (6) This operation increases the colormap size by the number of
  *          different gray (non-black or non-white) colors in the
  *          input colormap.  If there is not enough room in the colormap
  *          for this expansion, it returns 1 (error), and the caller
- *          should check the return value.  If an error is returned
- *          and the cmap is only 2 or 4 bpp, the pix can be converted
- *          to 8 bpp and this function will succeed if run again on
- *          a larger colormap.
- *      (6) Using the darkness of each original pixel in the rect,
+ *          should check the return value.
+ *      (7) Using the darkness of each original pixel in the rect,
  *          it generates a new color (based on the input rgb values).
  *          If type == L_PAINT_LIGHT, the new color is a (generally)
  *          darken-to-black version of the  input rgb color, where the
@@ -325,13 +323,10 @@ pixColorGrayCmap(PIX     *pixs,
                  l_int32  gval,
                  l_int32  bval)
 {
-l_int32    i, j, w, h, d, x1, y1, x2, y2, bw, bh, wpl;
-l_int32    val, nval;
-l_int32   *map;
-l_uint32  *line, *data;
-NUMA      *na;
-PIX       *pixt;
-PIXCMAP   *cmap, *cmapc;
+l_int32   w, h, d, ret;
+PIX      *pixt;
+BOXA     *boxa;
+PIXCMAP  *cmap;
 
     PROCNAME("pixColorGrayCmap");
 
@@ -339,84 +334,30 @@ PIXCMAP   *cmap, *cmapc;
         return ERROR_INT("pixs not defined", procName, 1);
     if ((cmap = pixGetColormap(pixs)) == NULL)
         return ERROR_INT("no colormap", procName, 1);
-    d = pixGetDepth(pixs);
+    pixGetDimensions(pixs, &w, &h, &d);
     if (d != 2 && d != 4 && d != 8)
         return ERROR_INT("depth not in {2, 4, 8}", procName, 1);
     if (type != L_PAINT_DARK && type != L_PAINT_LIGHT)
         return ERROR_INT("invalid type", procName, 1);
 
-        /* If 2 bpp or 4 bpp, see if the new colors will fit into
-         * the existing colormap.  If not, convert in-place to 8 bpp. */
+        /* If 2 bpp or 4 bpp, convert in-place to 8 bpp. */
     if (d == 2 || d == 4) {
-        cmapc = pixcmapCopy(cmap);  /* experiment with a copy */
-        if (addColorizedGrayToCmap(cmapc, type, rval, gval, bval, NULL)) {
-            pixt = pixConvertTo8(pixs, 1);
-            pixTransferAllData(pixs, &pixt, 0, 0);
-        }
-        pixcmapDestroy(&cmapc);
+        pixt = pixConvertTo8(pixs, 1);
+        pixTransferAllData(pixs, &pixt, 0, 0);
     }
 
-        /* Find gray colors, add the corresponding new colors,
-         * and set up a mapping table from gray to new.
-         * That table has the value 256 for all colors that are
-         * not to be mapped. */
-    cmap = pixGetColormap(pixs);
-    if (addColorizedGrayToCmap(cmap, type, rval, gval, bval, &na))
-        return ERROR_INT("no room; cmap full", procName, 1);
-    if ((map = numaGetIArray(na)) == NULL) {
-        numaDestroy(&na);
-        return ERROR_INT("map not made", procName, 1);
-    }
-
-        /* Determine the region of substitution */
-    pixGetDimensions(pixs, &w, &h, &d);  /* d may be different */
-    data = pixGetData(pixs);
-    wpl = pixGetWpl(pixs);
-    if (!box) {
-        x1 = y1 = 0;
-        x2 = w;
-        y2 = h;
+        /* If box == NULL, color the entire image */
+    boxa = boxaCreate(1);
+    if (box) {
+        boxaAddBox(boxa, box, L_COPY);
     } else {
-        boxGetGeometry(box, &x1, &y1, &bw, &bh);
-        x2 = x1 + bw - 1;
-        y2 = y1 + bh - 1;
+        box = boxCreate(0, 0, w, h);
+        boxaAddBox(boxa, box, L_INSERT);
     }
+    ret = pixColorGrayRegionsCmap(pixs, boxa, type, rval, gval, bval);
 
-        /* Remap gray pixels in the region */
-    for (i = y1; i <= y2; i++) {
-        if (i < 0 || i >= h)  /* clip */
-            continue;
-        line = data + i * wpl;
-        for (j = x1; j <= x2; j++) {
-            if (j < 0 || j >= w)  /* clip */
-                continue;
-            switch (d)
-            {
-            case 2:
-                val = GET_DATA_DIBIT(line, j);
-                nval = map[val];
-                if (nval != 256)
-                    SET_DATA_DIBIT(line, j, nval);
-                break;
-            case 4:
-                val = GET_DATA_QBIT(line, j);
-                nval = map[val];
-                if (nval != 256)
-                    SET_DATA_QBIT(line, j, nval);
-                break;
-            case 8:
-                val = GET_DATA_BYTE(line, j);
-                nval = map[val];
-                if (nval != 256)
-                    SET_DATA_BYTE(line, j, nval);
-                break;
-            }
-        }
-    }
-
-    FREE(map);
-    numaDestroy(&na);
-    return 0;
+    boxaDestroy(&boxa);
+    return ret;
 }
 
 
