@@ -34,9 +34,9 @@
  *          l_int32          pixWriteStreamPnm()
  *          l_int32          pixWriteStreamAsciiPnm()
  *
- *      Read/write to memory   [not on windows]
+ *      Read/write to memory
  *          PIX             *pixReadMemPnm()
- *          l_int32          sreadHeaderPnm()
+ *          l_int32          readHeaderMemPnm()
  *          l_int32          pixWriteMemPnm()
  *
  *      Local helpers
@@ -76,6 +76,11 @@
  *      with image processing before you write it out to file.
  *      The I/O routines just try to make the closest connection
  *      possible between the file and the Pix in memory.
+ *
+ *      On systems like windows without fmemopen() and open_memstream(),
+ *      we write data to a temp file and read it back for operations
+ *      between pix and compressed-data, such as pixReadMemPnm() and
+ *      pixWriteMemPnm().
  */
 
 #include <string.h>
@@ -119,7 +124,9 @@ PIX       *pix;
     if (!fp)
         return (PIX *)ERROR_PTR("fp not defined", procName, NULL);
 
-    if (freadHeaderPnm(fp, &pix, &w, &h, &d, &type, NULL, NULL))
+    if (freadHeaderPnm(fp, &w, &h, &d, &type, NULL, NULL))
+        return (PIX *)ERROR_PTR( "header read failed", procName, NULL);
+    if ((pix = pixCreate(w, h, d)) == NULL)
         return (PIX *)ERROR_PTR( "pix not made", procName, NULL);
     data = pixGetData(pix);
     wpl = pixGetWpl(pix);
@@ -132,8 +139,7 @@ PIX       *pix;
                     if (pnmReadNextAsciiValue(fp, &val))
                         return (PIX *)ERROR_PTR( "read abend", procName, pix);
                     pixSetPixel(pix, j, i, val);
-                }
-                else {  /* type == 3 */
+                } else {  /* type == 3 */
                     if (pnmReadNextAsciiValue(fp, &rval))
                         return (PIX *)ERROR_PTR( "read abend", procName, pix);
                     if (pnmReadNextAsciiValue(fp, &gval))
@@ -178,8 +184,7 @@ PIX       *pix;
                     else  /* d == 8 */
                         SET_DATA_BYTE(line, j, val8);
                 }
-            }
-            else {  /* d == 16 */
+            } else {  /* d == 16 */
                 for (j = 0; j < w; j++) {
                     if (fread(&val16, 2, 1, fp) != 1)
                         return (PIX *)ERROR_PTR( "16 bpp error", procName, pix);
@@ -212,37 +217,40 @@ PIX       *pix;
  *  readHeaderPnm()
  *
  *      Input:  filename
- *              &pix (<optional return> use null to return only header data)
- *              &width (<return>)
- *              &height (<return>)
- *              &depth (<return>)
- *              &type (<return> pnm type)
+ *              &w (<optional return>)
+ *              &h (<optional return>)
+ *              &d (<optional return>)
+ *              &type (<optional return> pnm type)
  *              &bps (<optional return>, bits/sample)
  *              &spp (<optional return>, samples/pixel)
  *      Return: 0 if OK, 1 on error
  */
 l_int32
 readHeaderPnm(const char *filename,
-              PIX     **ppix,
-              l_int32  *pwidth,
-              l_int32  *pheight,
-              l_int32  *pdepth,
-              l_int32  *ptype,
-              l_int32  *pbps,
-              l_int32  *pspp)
+              l_int32    *pw,
+              l_int32    *ph,
+              l_int32    *pd,
+              l_int32    *ptype,
+              l_int32    *pbps,
+              l_int32    *pspp)
 {
 l_int32  ret;
 FILE    *fp;
 
     PROCNAME("readHeaderPnm");
 
+    if (pw) *pw = 0;
+    if (ph) *ph = 0;
+    if (pd) *pd = 0;
+    if (ptype) *ptype = 0;
+    if (pbps) *pbps = 0;
+    if (pspp) *pspp = 0;
     if (!filename)
         return ERROR_INT("filename not defined", procName, 1);
-    if (!pwidth || !pheight || !pbps || !pspp)
-        return ERROR_INT("input ptr(s) not defined", procName, 1);
+
     if ((fp = fopenReadStream(filename)) == NULL)
         return ERROR_INT("image file not found", procName, 1);
-    ret = freadHeaderPnm(fp, ppix, pwidth, pheight, pdepth, ptype, pbps, pspp);
+    ret = freadHeaderPnm(fp, pw, ph, pd, ptype, pbps, pspp);
     fclose(fp);
     return ret;
 }
@@ -252,21 +260,19 @@ FILE    *fp;
  *  freadHeaderPnm()
  *
  *      Input:  stream opened for read
- *              &pix (<optional return> use null to return only header data)
- *              &width (<return>)
- *              &height (<return>)
- *              &depth (<return>)
- *              &type (<return> pnm type)
+ *              &w (<optional return>)
+ *              &h (<optional return>)
+ *              &d (<optional return>)
+ *              &type (<optional return> pnm type)
  *              &bps (<optional return>, bits/sample)
  *              &spp (<optional return>, samples/pixel)
  *      Return: 0 if OK, 1 on error
  */
 l_int32
 freadHeaderPnm(FILE     *fp,
-               PIX     **ppix,
-               l_int32  *pwidth,
-               l_int32  *pheight,
-               l_int32  *pdepth,
+               l_int32  *pw,
+               l_int32  *ph,
+               l_int32  *pd,
                l_int32  *ptype,
                l_int32  *pbps,
                l_int32  *pspp)
@@ -276,10 +282,14 @@ l_int32  maxval;
 
     PROCNAME("freadHeaderPnm");
 
+    if (pw) *pw = 0;
+    if (ph) *ph = 0;
+    if (pd) *pd = 0;
+    if (ptype) *ptype = 0;
+    if (pbps) *pbps = 0;
+    if (pspp) *pspp = 0;
     if (!fp)
         return ERROR_INT("fp not defined", procName, 1);
-    if (!pwidth || !pheight || !pdepth || !ptype)
-        return ERROR_INT("input ptr(s) not defined", procName, 1);
 
     if (fscanf(fp, "P%d\n", &type) != 1)
         return ERROR_INT("invalid read for type", procName, 1);
@@ -292,46 +302,39 @@ l_int32  maxval;
     if (fscanf(fp, "%d %d\n", &w, &h) != 2)
         return ERROR_INT("invalid read for w,h", procName, 1);
     if (w <= 0 || h <= 0 || w > MAX_PNM_WIDTH || h > MAX_PNM_HEIGHT)
-        return ERROR_INT("invalid sizes", procName, 1);
+        return ERROR_INT("invalid size", procName, 1);
 
         /* Get depth of pix */
-    if (type == 1 || type == 4)
+    if (type == 1 || type == 4) {
         d = 1;
-    else if (type == 2 || type == 5) {
+    } else if (type == 2 || type == 5) {
         if (fscanf(fp, "%d\n", &maxval) != 1)
             return ERROR_INT("invalid read for maxval (2,5)", procName, 1);
-        if (maxval == 3)
+        if (maxval == 3) {
             d = 2;
-        else if (maxval == 15)
+        } else if (maxval == 15) {
             d = 4;
-        else if (maxval == 255)
+        } else if (maxval == 255) {
             d = 8;
-        else if (maxval == 0xffff)
+        } else if (maxval == 0xffff) {
             d = 16;
-        else {
+        } else {
             fprintf(stderr, "maxval = %d\n", maxval);
             return ERROR_INT("invalid maxval", procName, 1);
         }
-    }
-    else {  /* type == 3 || type == 6; this is rgb  */
+    } else {  /* type == 3 || type == 6; this is rgb  */
         if (fscanf(fp, "%d\n", &maxval) != 1)
             return ERROR_INT("invalid read for maxval (3,6)", procName, 1);
         if (maxval != 255)
-            L_WARNING_INT("unexpected maxval = %d", procName, maxval);
+            L_WARNING("unexpected maxval = %d\n", procName, maxval);
         d = 32;
     }
-    *pwidth = w;
-    *pheight = h;
-    *pdepth = d;
-    *ptype = type;
+    if (pw) *pw = w;
+    if (ph) *ph = h;
+    if (pd) *pd = d;
+    if (ptype) *ptype = type;
     if (pbps) *pbps = (d == 32) ? 8 : d;
     if (pspp) *pspp = (d == 32) ? 3 : 1;
-
-    if (!ppix)
-        return 0;
-
-    if ((*ppix = pixCreate(w, h, d)) == NULL)  /* return pix initialized to 0 */
-        return ERROR_INT( "pix not made", procName, 1);
     return 0;
 }
 
@@ -395,8 +398,7 @@ PIX       *pixs;
                 fwrite(&val8, 1, 1, fp);
             }
         }
-    }
-    else if (ds == 2 || ds == 4 || ds == 8 || ds == 16) {  /* grayscale */
+    } else if (ds == 2 || ds == 4 || ds == 8 || ds == 16) {  /* grayscale */
         maxval = (1 << ds) - 1;
         fprintf(fp, "P5\n# Raw PGM file written by leptonica "
                     "(www.leptonica.com)\n%d %d\n%d\n", w, h, maxval);
@@ -414,8 +416,7 @@ PIX       *pixs;
                     fwrite(&val8, 1, 1, fp);
                 }
             }
-        }
-        else {  /* ds == 16 */
+        } else {  /* ds == 16 */
             for (i = 0; i < h; i++) {
                 lines = datas + i * wpls;
                 for (j = 0; j < w; j++) {
@@ -424,8 +425,7 @@ PIX       *pixs;
                 }
             }
         }
-    }
-    else {  /* rgb color */
+    } else {  /* rgb color */
         fprintf(fp, "P6\n# Raw PPM file written by leptonica "
                     "(www.leptonica.com)\n%d %d\n255\n", w, h);
 
@@ -436,16 +436,15 @@ PIX       *pixs;
                 if (fwrite(lines, 1, filebpl, fp) != filebpl)
                     writeerror = 1;
             }
-        }
-        else {  /* 32 bpp rgb */
+        } else {  /* 32 bpp rgb */
             for (i = 0; i < h; i++) {
                 lines = datas + i * wpls;
                 for (j = 0; j < wpls; j++) {
                     pword = lines + j;
-                    pel[0] = *((l_uint8 *)pword + 3);   /* red   */
-                    pel[1] = *((l_uint8 *)pword + 2);   /* green */
-                    pel[2] = *((l_uint8 *)pword + 1);   /* blue  */
-                    if (fwrite(&pel, 1, 3, fp) != 3)
+                    pel[0] = GET_DATA_BYTE(pword, COLOR_RED);
+                    pel[1] = GET_DATA_BYTE(pword, COLOR_GREEN);
+                    pel[2] = GET_DATA_BYTE(pword, COLOR_BLUE);
+                    if (fwrite(pel, 1, 3, fp) != 3)
                         writeerror = 1;
                 }
             }
@@ -517,8 +516,7 @@ PIX       *pixs;
                     fputc('\n', fp);
             }
         }
-    }
-    else if (ds == 2 || ds == 4 || ds == 8 || ds == 16) {  /* grayscale */
+    } else if (ds == 2 || ds == 4 || ds == 8 || ds == 16) {  /* grayscale */
         maxval = (1 << ds) - 1;
         fprintf(fp, "P2\n# Ascii PGM file written by leptonica "
                     "(www.leptonica.com)\n%d %d\n%d\n", w, h, maxval);
@@ -531,18 +529,15 @@ PIX       *pixs;
                     sprintf(buffer, "%1d ", val);
                     fwrite(buffer, 1, 2, fp);
                     count += 2;
-                }
-                else if (ds == 4) {
+                } else if (ds == 4) {
                     sprintf(buffer, "%2d ", val);
                     fwrite(buffer, 1, 3, fp);
                     count += 3;
-                }
-                else if (ds == 8) {
+                } else if (ds == 8) {
                     sprintf(buffer, "%3d ", val);
                     fwrite(buffer, 1, 4, fp);
                     count += 4;
-                }
-                else {  /* ds == 16 */
+                } else {  /* ds == 16 */
                     sprintf(buffer, "%5d ", val);
                     fwrite(buffer, 1, 6, fp);
                     count += 6;
@@ -553,11 +548,9 @@ PIX       *pixs;
                 }
             }
         }
-    }
-    else {  /* rgb color */
+    } else {  /* rgb color */
         fprintf(fp, "P3\n# Ascii PPM file written by leptonica "
                     "(www.leptonica.com)\n%d %d\n255\n", w, h);
-
         count = 0;
         for (i = 0; i < h; i++) {
             for (j = 0; j < w; j++) {
@@ -591,9 +584,9 @@ PIX       *pixs;
 #endif  /* HAVE_CONFIG_H */
 
 #if HAVE_FMEMOPEN
-
 extern FILE *open_memstream(char **data, size_t *size);
 extern FILE *fmemopen(void *data, size_t size, const char *mode);
+#endif  /* HAVE_FMEMOPEN */
 
 /*!
  *  pixReadMemPnm()
@@ -609,60 +602,71 @@ PIX *
 pixReadMemPnm(const l_uint8  *cdata,
               size_t          size)
 {
-l_uint8  *data;
-FILE     *fp;
-PIX      *pix;
+FILE  *fp;
+PIX   *pix;
 
     PROCNAME("pixReadMemPnm");
 
     if (!cdata)
         return (PIX *)ERROR_PTR("cdata not defined", procName, NULL);
 
-    data = (l_uint8 *)cdata;  /* we're really not going to change this */
-    if ((fp = fmemopen(data, size, "r")) == NULL)
+#if HAVE_FMEMOPEN
+    if ((fp = fmemopen((l_uint8 *)cdata, size, "r")) == NULL)
         return (PIX *)ERROR_PTR("stream not opened", procName, NULL);
+#else
+    L_WARNING("work-around: writing to a temp file\n", procName);
+    fp = tmpfile();
+    fwrite(cdata, 1, size, fp);
+    rewind(fp);
+#endif  /* HAVE_FMEMOPEN */
     pix = pixReadStreamPnm(fp);
     fclose(fp);
+    if (!pix) L_ERROR("pix not read\n", procName);
     return pix;
 }
 
 
 /*!
- *  sreadHeaderPnm()
+ *  readHeaderMemPnm()
  *
  *      Input:  cdata (const; pnm-encoded)
  *              size (of data)
- *              &width (<return>)
- *              &height (<return>)
- *              &depth (<return>)
- *              &type (<return> pnm type)
+ *              &w (<optional return>)
+ *              &h (<optional return>)
+ *              &d (<optional return>)
+ *              &type (<optional return> pnm type)
  *              &bps (<optional return>, bits/sample)
  *              &spp (<optional return>, samples/pixel)
  *      Return: 0 if OK, 1 on error
  */
 l_int32
-sreadHeaderPnm(const l_uint8  *cdata,
-               size_t          size,
-               l_int32        *pwidth,
-               l_int32        *pheight,
-               l_int32        *pdepth,
-               l_int32        *ptype,
-               l_int32        *pbps,
-               l_int32        *pspp)
+readHeaderMemPnm(const l_uint8  *cdata,
+                 size_t          size,
+                 l_int32        *pw,
+                 l_int32        *ph,
+                 l_int32        *pd,
+                 l_int32        *ptype,
+                 l_int32        *pbps,
+                 l_int32        *pspp)
 {
-l_int32   ret;
-l_uint8  *data;
-FILE     *fp;
+l_int32  ret;
+FILE    *fp;
 
-    PROCNAME("sreadHeaderPnm");
+    PROCNAME("readHeaderMemPnm");
 
     if (!cdata)
         return ERROR_INT("cdata not defined", procName, 1);
 
-    data = (l_uint8 *)cdata;  /* we're really not going to change this */
-    if ((fp = fmemopen(data, size, "r")) == NULL)
+#if HAVE_FMEMOPEN
+    if ((fp = fmemopen((l_uint8 *)cdata, size, "r")) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
-    ret = freadHeaderPnm(fp, NULL, pwidth, pheight, pdepth, ptype, pbps, pspp);
+#else
+    L_WARNING("work-around: writing to a temp file\n", procName);
+    fp = tmpfile();
+    fwrite(cdata, 1, size, fp);
+    rewind(fp);
+#endif  /* HAVE_FMEMOPEN */
+    ret = freadHeaderPnm(fp, pw, ph, pd, ptype, pbps, pspp);
     fclose(fp);
     if (ret)
         return ERROR_INT("header data read failed", procName, 1);
@@ -692,6 +696,8 @@ FILE    *fp;
 
     PROCNAME("pixWriteMemPnm");
 
+    if (pdata) *pdata = NULL;
+    if (psize) *psize = 0;
     if (!pdata)
         return ERROR_INT("&data not defined", procName, 1 );
     if (!psize)
@@ -699,53 +705,21 @@ FILE    *fp;
     if (!pix)
         return ERROR_INT("&pix not defined", procName, 1 );
 
+#if HAVE_FMEMOPEN
     if ((fp = open_memstream((char **)pdata, psize)) == NULL)
         return ERROR_INT("stream not opened", procName, 1);
     ret = pixWriteStreamPnm(fp, pix);
+#else
+    L_WARNING("work-around: writing to a temp file\n", procName);
+    fp = tmpfile();
+    ret = pixWriteStreamPnm(fp, pix);
+    rewind(fp);
+    *pdata = l_binaryReadStream(fp, psize);
+#endif  /* HAVE_FMEMOPEN */
     fclose(fp);
     return ret;
 }
 
-#else
-
-PIX *
-pixReadMemPnm(const l_uint8  *cdata,
-              size_t          size)
-{
-    return (PIX *)ERROR_PTR(
-        "pnm read from memory not implemented on this platform",
-        "pixReadMemPnm", NULL);
-}
-
-
-l_int32
-sreadHeaderPnm(const l_uint8  *cdata,
-               size_t          size,
-               l_int32        *pwidth,
-               l_int32        *pheight,
-               l_int32        *pdepth,
-               l_int32        *ptype,
-               l_int32        *pbps,
-               l_int32        *pspp)
-{
-    return ERROR_INT(
-        "pnm read header from memory not implemented on this platform",
-        "sreadHeaderPnm", 1);
-}
-
-
-l_int32
-pixWriteMemPnm(l_uint8  **pdata,
-               size_t    *psize,
-               PIX       *pix)
-{
-    return ERROR_INT(
-        "pnm write to memory not implemented on this platform",
-        "pixWriteMemPnm", 1);
-}
-
-#endif  /* HAVE_FMEMOPEN */
-
 
 /*--------------------------------------------------------------------*
  *                          Static helpers                            *
@@ -759,18 +733,18 @@ pixWriteMemPnm(l_uint8  **pdata,
  *      (1) This reads the next sample value in ascii from the the file.
  */
 static l_int32
-pnmReadNextAsciiValue(FILE  *fp,
-                      l_int32 *pval)
+pnmReadNextAsciiValue(FILE     *fp,
+                      l_int32  *pval)
 {
 l_int32   c, ignore;
 
     PROCNAME("pnmReadNextAsciiValue");
 
-    if (!fp)
-        return ERROR_INT("stream not open", procName, 1);
     if (!pval)
         return ERROR_INT("&val not defined", procName, 1);
     *pval = 0;
+    if (!fp)
+        return ERROR_INT("stream not open", procName, 1);
     do {  /* skip whitespace */
         if ((c = fgetc(fp)) == EOF)
             return 1;
