@@ -56,17 +56,12 @@
  *          l_int32              convertJpegToPS()
  *          l_int32              convertJpegToPSString()
  *          char                *generateJpegPS()
- *          L_COMPRESSED_DATA   *pixGenerateJpegData()
- *          L_COMPRESSED_DATA   *l_generateJpegData()
- *          void                 compressed_dataDestroy()
  *
  *     For g4 fax compressed images (use ccitt g4 compression)
  *          l_int32              convertG4ToPSEmbed()
  *          l_int32              convertG4ToPS()
  *          l_int32              convertG4ToPSString()
  *          char                *generateG4PS()
- *          L_COMPRESSED_DATA   *pixGenerateG4Data()
- *          L_COMPRESSED_DATA   *l_generateG4Data()
  *
  *     For multipage tiff images
  *          l_int32              convertTiffMultipageToPS()
@@ -76,8 +71,6 @@
  *          l_int32              convertFlateToPS()
  *          l_int32              convertFlateToPSString()
  *          char                *generateFlatePS()
- *          L_COMPRESSED_DATA   *l_generateFlateData()
- *          L_COMPRESSED_DATA   *pixGenerateFlateData()
  *
  *     Write to memory
  *          l_int32              pixWriteMemPS()
@@ -107,9 +100,7 @@
     /* Set default for writing bounding box hint */
 static l_int32  var_PS_WRITE_BOUNDING_BOX = 1;
 
-    /* MS VC++ can't handle array initialization with static consts ! */
-#define L_BUF_SIZE      512
-
+static const l_int32  L_BUF_SIZE = 512;
 static const l_int32  DEFAULT_INPUT_RES   = 300;  /* typical scan res, ppi */
 static const l_int32  MIN_RES             = 5;
 static const l_int32  MAX_RES             = 3000;
@@ -366,8 +357,7 @@ PIX       *pix;
                 hexdata[k++] = nib2;
             }
         }
-    }
-    else  {  /* d == 32; hexdata bytes packed RGBRGB..., 2 per sample */
+    } else  {  /* d == 32; hexdata bytes packed RGBRGB..., 2 per sample */
         for (i = 0, k = 0; i < h; i++) {
             line = data + i * wpl;
             for (j = 0; j < w; j++) {
@@ -445,9 +435,9 @@ SARRAY  *sa;
             "%%%%BoundingBox: %7.2f %7.2f %7.2f %7.2f",
             xpt, ypt, xpt + wpt, ypt + hpt);
         sarrayAddString(sa, bigbuf, L_COPY);
-    }
-    else    /* boxflag == 1 */
+    } else {  /* boxflag == 1 */
         sarrayAddString(sa, (char *)"gsave", L_COPY);
+    }
 
     if (d == 1)
         sarrayAddString(sa,
@@ -477,8 +467,7 @@ SARRAY  *sa;
             sarrayAddString(sa,
               (char *)"{currentfile bpl readhexstring pop} false 3 colorimage",
               L_COPY);
-    }
-    else {  /* boxflag == 1 */
+    } else {  /* boxflag == 1 */
         if (d == 1 || d == 8)
             sarrayAddString(sa,
               (char *)"{currentfile bpl readhexstring pop} bind image", L_COPY);
@@ -553,8 +542,8 @@ l_float32  winch, hinch, xinch, yinch, fres;
 
         /* Limit valid resolution interval */
     if (res < MIN_RES || res > MAX_RES) {
-        L_WARNING_INT("res %d out of bounds; using default res; no scaling",
-                      procName, res);
+        L_WARNING("res %d out of bounds; using default res; no scaling\n",
+                  procName, res);
         res = DEFAULT_INPUT_RES;
         fres = (l_float32)res;
     }
@@ -564,8 +553,7 @@ l_float32  winch, hinch, xinch, yinch, fres;
         hinch = (l_float32)hpix / fres;
         xinch = (8.5 - winch) / 2.;
         yinch = (11.0 - hinch) / 2.;
-    }
-    else {
+    } else {
         boxGetGeometry(box, &bx, &by, &bw, &bh);
         if (bw == 0)
             winch = (l_float32)wpix / fres;
@@ -580,13 +568,13 @@ l_float32  winch, hinch, xinch, yinch, fres;
     }
 
     if (xinch < 0)
-        L_WARNING("left edge < 0.0 inch", procName);
+        L_WARNING("left edge < 0.0 inch\n", procName);
     if (xinch + winch > 8.5)
-        L_WARNING("right edge > 8.5 inch", procName);
+        L_WARNING("right edge > 8.5 inch\n", procName);
     if (yinch < 0.0)
-        L_WARNING("bottom edge < 0.0 inch", procName);
+        L_WARNING("bottom edge < 0.0 inch\n", procName);
     if (yinch + hinch > 11.0)
-        L_WARNING("top edge > 11.0 inch", procName);
+        L_WARNING("top edge > 11.0 inch\n", procName);
 
     *pwpt = 72. * winch;
     *phpt = 72. * hinch;
@@ -647,10 +635,10 @@ l_int32
 convertJpegToPSEmbed(const char  *filein,
                      const char  *fileout)
 {
-char               *outstr;
-l_int32             w, h, nbytes;
-l_float32           xpt, ypt, wpt, hpt;
-L_COMPRESSED_DATA  *cid;
+char         *outstr;
+l_int32       w, h, nbytes;
+l_float32     xpt, ypt, wpt, hpt;
+L_COMP_DATA  *cid;
 
     PROCNAME("convertJpegToPSEmbed");
 
@@ -672,8 +660,7 @@ L_COMPRESSED_DATA  *cid;
     if (w * 11.0 > h * 8.5) {
         wpt = 572.0;   /* 612 - 2 * 20 */
         hpt = wpt * (l_float32)h / (l_float32)w;
-    }
-    else {
+    } else {
         hpt = 752.0;   /* 792 - 2 * 20 */
         wpt = hpt * (l_float32)w / (l_float32)h;
     }
@@ -688,7 +675,7 @@ L_COMPRESSED_DATA  *cid;
     if (l_binaryWrite(fileout, "w", outstr, nbytes))
         return ERROR_INT("ps string not written to file", procName, 1);
     FREE(outstr);
-    compressed_dataDestroy(&cid);
+    l_CIDataDestroy(&cid);
     return 0;
 }
 
@@ -825,9 +812,9 @@ convertJpegToPSString(const char  *filein,
                       l_int32      pageno,
                       l_int32      endpage)
 {
-char               *outstr;
-l_float32           xpt, ypt, wpt, hpt;
-L_COMPRESSED_DATA  *cid;
+char         *outstr;
+l_float32     xpt, ypt, wpt, hpt;
+L_COMP_DATA  *cid;
 
     PROCNAME("convertJpegToPSString");
 
@@ -872,8 +859,8 @@ L_COMPRESSED_DATA  *cid;
 #if  DEBUG_JPEG
     fprintf(stderr, "w = %d, h = %d, bps = %d, spp = %d\n",
             cid->w, cid->h, cid->bps, cid->spp);
-    fprintf(stderr, "comp bytes = %d, nbytes85 = %d, ratio = %5.3f\n",
-           cid->nbytescomp, cid->nbytes85,
+    fprintf(stderr, "comp bytes = %ld, nbytes85 = %ld, ratio = %5.3f\n",
+            (unsigned long)cid->nbytescomp, (unsigned long)cid->nbytes85,
            (l_float32)cid->nbytes85 / (l_float32)cid->nbytescomp);
     fprintf(stderr, "xpt = %7.2f, ypt = %7.2f, wpt = %7.2f, hpt = %7.2f\n",
              xpt, ypt, wpt, hpt);
@@ -885,7 +872,7 @@ L_COMPRESSED_DATA  *cid;
         return ERROR_INT("outstr not made", procName, 1);
     *poutstr = outstr;
     *pnbytes = strlen(outstr);
-    compressed_dataDestroy(&cid);
+    l_CIDataDestroy(&cid);
     return 0;
 }
 
@@ -909,14 +896,14 @@ L_COMPRESSED_DATA  *cid;
  *      (1) Low-level function.
  */
 char *
-generateJpegPS(const char         *filein,
-               L_COMPRESSED_DATA  *cid,
-               l_float32           xpt,
-               l_float32           ypt,
-               l_float32           wpt,
-               l_float32           hpt,
-               l_int32             pageno,
-               l_int32             endpage)
+generateJpegPS(const char   *filein,
+               L_COMP_DATA  *cid,
+               l_float32     xpt,
+               l_float32     ypt,
+               l_float32     wpt,
+               l_float32     hpt,
+               l_int32       pageno,
+               l_int32       endpage)
 {
 l_int32  w, h, bps, spp;
 char    *outstr;
@@ -1013,157 +1000,6 @@ SARRAY  *sa;
 }
 
 
-/*!
- *  pixGenerateJpegData()
- *
- *      Input:  pixs (8 or 32 bpp, no colormap)
- *              ascii85flag (0 for jpeg; 1 for ascii85-encoded jpeg)
- *              quality (0 for default, which is 75)
- *      Return: cid (jpeg compressed data), or null on error
- *
- *  Notes:
- *      (1) Set ascii85flag:
- *           - 0 for binary data (not permitted in PostScript)
- *           - 1 for ascii85 (5 for 4) encoded binary data
- */
-L_COMPRESSED_DATA *
-pixGenerateJpegData(PIX     *pixs,
-                    l_int32  ascii85flag,
-                    l_int32  quality)
-{
-l_int32             d;
-char               *tname;
-L_COMPRESSED_DATA  *cid;
-
-    PROCNAME("pixGenerateJpegData");
-
-    if (!pixs)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("pixs not defined",
-                                              procName, NULL);
-    if (pixGetColormap(pixs))
-        return (L_COMPRESSED_DATA *)ERROR_PTR("pixs has colormap",
-                                              procName, NULL);
-    d = pixGetDepth(pixs);
-    if (d != 8 && d != 32)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("pixs not 8 or 32 bpp",
-                                              procName, NULL);
-
-        /* Compress to a temp jpeg file */
-    tname = genTempFilename("/tmp", "temp.jpg", 1, 1);
-    pixWriteJpeg(tname, pixs, quality, 0);
-
-    cid = l_generateJpegData(tname, ascii85flag);
-    FREE(tname);
-    return cid;
-}
-
-
-/*!
- *  l_generateJpegData()
- *
- *      Input:  fname (of jpeg file)
- *              ascii85flag (0 for jpeg; 1 for ascii85-encoded jpeg)
- *      Return: cid (containing jpeg data), or null on error
- *
- *  Notes:
- *      (1) Set ascii85flag:
- *           - 0 for binary data (not permitted in PostScript)
- *           - 1 for ascii85 (5 for 4) encoded binary data
- */
-L_COMPRESSED_DATA *
-l_generateJpegData(const char  *fname,
-                   l_int32      ascii85flag)
-{
-l_uint8            *datacomp = NULL;  /* entire jpeg compressed file */
-char               *data85 = NULL;  /* ascii85 encoded jpeg compressed file */
-l_int32             w, h, xres, yres, bps, spp;
-l_int32             nbytes85;
-size_t              nbytescomp;
-FILE               *fp;
-L_COMPRESSED_DATA  *cid;
-
-    PROCNAME("l_generateJpegData");
-
-    if (!fname)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("fname not defined",
-                                              procName, NULL);
-
-        /* The returned jpeg data in memory is the entire jpeg file,
-         * which starts with ffd8 and ends with ffd9 */
-    if ((datacomp = l_binaryRead(fname, &nbytescomp)) == NULL)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("datacomp not extracted",
-                                              procName, NULL);
-
-        /* Read the metadata */
-    if ((fp = fopenReadStream(fname)) == NULL)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("stream not opened",
-                                              procName, NULL);
-    freadHeaderJpeg(fp, &w, &h, &spp, NULL, NULL);
-    bps = 8;
-    fgetJpegResolution(fp, &xres, &yres);
-    fclose(fp);
-
-        /* Optionally, encode the compressed data */
-    if (ascii85flag == 1) {
-        data85 = encodeAscii85(datacomp, nbytescomp, &nbytes85);
-        FREE(datacomp);
-        if (!data85)
-            return (L_COMPRESSED_DATA *)ERROR_PTR("data85 not made",
-                                                  procName, NULL);
-        else
-            data85[nbytes85 - 1] = '\0';  /* remove the newline */
-    }
-
-    cid = (L_COMPRESSED_DATA *)CALLOC(1, sizeof(L_COMPRESSED_DATA));
-    if (!cid)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("cid not made", procName, NULL);
-    if (ascii85flag == 0)
-        cid->datacomp = datacomp;
-    else {  /* ascii85 */
-        cid->data85 = data85;
-        cid->nbytes85 = nbytes85;
-    }
-    cid->type = L_JPEG_ENCODE;
-    cid->nbytescomp = nbytescomp;
-    cid->w = w;
-    cid->h = h;
-    cid->bps = bps;
-    cid->spp = spp;
-    cid->res = xres;
-    return cid;
-}
-
-
-/*!
- *  compressed_dataDestroy()
- *
- *      Input:  &cid (<will be set to null before returning>)
- *      Return: void
- */
-void
-compressed_dataDestroy(L_COMPRESSED_DATA  **pcid)
-{
-L_COMPRESSED_DATA  *cid;
-
-    PROCNAME("compressed_dataDestroy");
-
-    if (pcid == NULL) {
-        L_WARNING("ptr address is null!", procName);
-        return;
-    }
-    if ((cid = *pcid) == NULL)
-        return;
-
-    if (cid->datacomp) FREE(cid->datacomp);
-    if (cid->data85) FREE(cid->data85);
-    if (cid->cmapdata85) FREE(cid->cmapdata85);
-    if (cid->cmapdatahex) FREE(cid->cmapdatahex);
-    FREE(cid);
-    *pcid = NULL;
-    return;
-}
-
-
 /*-------------------------------------------------------------*
  *                  For ccitt g4 compressed images             *
  *-------------------------------------------------------------*/
@@ -1188,10 +1024,10 @@ l_int32
 convertG4ToPSEmbed(const char  *filein,
                    const char  *fileout)
 {
-char               *outstr;
-l_int32             w, h, nbytes;
-l_float32           xpt, ypt, wpt, hpt;
-L_COMPRESSED_DATA  *cid;
+char         *outstr;
+l_int32       w, h, nbytes;
+l_float32     xpt, ypt, wpt, hpt;
+L_COMP_DATA  *cid;
 
     PROCNAME("convertG4ToPSEmbed");
 
@@ -1212,8 +1048,7 @@ L_COMPRESSED_DATA  *cid;
     if (w * 11.0 > h * 8.5) {
         wpt = 572.0;   /* 612 - 2 * 20 */
         hpt = wpt * (l_float32)h / (l_float32)w;
-    }
-    else {
+    } else {
         hpt = 752.0;   /* 792 - 2 * 20 */
         wpt = hpt * (l_float32)w / (l_float32)h;
     }
@@ -1228,7 +1063,7 @@ L_COMPRESSED_DATA  *cid;
     if (l_binaryWrite(fileout, "w", outstr, nbytes))
         return ERROR_INT("ps string not written to file", procName, 1);
     FREE(outstr);
-    compressed_dataDestroy(&cid);
+    l_CIDataDestroy(&cid);
     return 0;
 }
 
@@ -1362,9 +1197,9 @@ convertG4ToPSString(const char  *filein,
                     l_int32      maskflag,
                     l_int32      endpage)
 {
-char               *outstr;
-l_float32           xpt, ypt, wpt, hpt;
-L_COMPRESSED_DATA  *cid;
+char         *outstr;
+l_float32     xpt, ypt, wpt, hpt;
+L_COMP_DATA  *cid;
 
     PROCNAME("convertG4ToPSString");
 
@@ -1386,9 +1221,9 @@ L_COMPRESSED_DATA  *cid;
     if (scale == 0.0)
         scale = 1.0;
     if (res <= 0) {
-        if (cid->res > 0)
+        if (cid->res > 0) {
             res = cid->res;
-        else {
+        } else {
             if (cid->h <= 3509)  /* A4 height at 300 ppi */
                 res = 300;
             else
@@ -1406,8 +1241,8 @@ L_COMPRESSED_DATA  *cid;
 #if  DEBUG_G4
     fprintf(stderr, "w = %d, h = %d, minisblack = %d\n",
             cid->w, cid->h, cid->minisblack);
-    fprintf(stderr, "comp bytes = %d, nbytes85 = %d\n",
-            cid->nbytescomp, cid->nbytes85);
+    fprintf(stderr, "comp bytes = %ld, nbytes85 = %ld\n",
+            (unsigned long)cid->nbytescomp, (unsigned long)cid->nbytes85);
     fprintf(stderr, "xpt = %7.2f, ypt = %7.2f, wpt = %7.2f, hpt = %7.2f\n",
              xpt, ypt, wpt, hpt);
 #endif   /* DEBUG_G4 */
@@ -1419,7 +1254,7 @@ L_COMPRESSED_DATA  *cid;
         return ERROR_INT("outstr not made", procName, 1);
     *poutstr = outstr;
     *pnbytes = strlen(outstr);
-    compressed_dataDestroy(&cid);
+    l_CIDataDestroy(&cid);
     return 0;
 }
 
@@ -1445,15 +1280,15 @@ L_COMPRESSED_DATA  *cid;
  *      (1) Low-level function.
  */
 char *
-generateG4PS(const char         *filein,
-             L_COMPRESSED_DATA  *cid,
-             l_float32           xpt,
-             l_float32           ypt,
-             l_float32           wpt,
-             l_float32           hpt,
-             l_int32             maskflag,
-             l_int32             pageno,
-             l_int32             endpage)
+generateG4PS(const char   *filein,
+             L_COMP_DATA  *cid,
+             l_float32     xpt,
+             l_float32     ypt,
+             l_float32     wpt,
+             l_float32     hpt,
+             l_int32       maskflag,
+             l_int32       pageno,
+             l_int32       endpage)
 {
 l_int32  w, h;
 char    *outstr;
@@ -1555,121 +1390,6 @@ SARRAY  *sa;
 }
 
 
-/*!
- *  pixGenerateG4Data()
- *
- *      Input:  pixs (1 bpp)
- *              ascii85flag (0 for gzipped; 1 for ascii85-encoded gzipped)
- *      Return: cid (g4 compressed image data), or null on error
- *
- *  Notes:
- *      (1) Set ascii85flag:
- *           - 0 for binary data (not permitted in PostScript)
- *           - 1 for ascii85 (5 for 4) encoded binary data
- */
-L_COMPRESSED_DATA *
-pixGenerateG4Data(PIX     *pixs,
-                  l_int32  ascii85flag)
-{
-char               *tname;
-L_COMPRESSED_DATA  *cid;
-
-    PROCNAME("pixGenerateG4Data");
-
-    if (!pixs)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("pixs not defined",
-                                              procName, NULL);
-    if (pixGetDepth(pixs) != 1)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("pixs not 1 bpp",
-                                              procName, NULL);
-
-        /* Compress to a temp tiff g4 file */
-    tname = genTempFilename("/tmp", "temp.tif", 1, 1);
-    pixWrite(tname, pixs, IFF_TIFF_G4);
-
-    cid = l_generateG4Data(tname, ascii85flag);
-    FREE(tname);
-    return cid;
-}
-
-
-/*!
- *  l_generateG4Data()
- *
- *      Input:  fname (of g4 compressed file)
- *              ascii85flag (0 for g4 compressed; 1 for ascii85-encoded g4)
- *      Return: cid (g4 compressed image data), or null on error
- *
- *  Notes:
- *      (1) Set ascii85flag:
- *           - 0 for binary data (not permitted in PostScript)
- *           - 1 for ascii85 (5 for 4) encoded binary data
- */
-L_COMPRESSED_DATA *
-l_generateG4Data(const char  *fname,
-                 l_int32      ascii85flag)
-{
-l_uint8            *datacomp = NULL;  /* g4 compressed raster data */
-char               *data85 = NULL;  /* ascii85 encoded g4 compressed data */
-l_int32             w, h, xres, yres;
-l_int32             minisblack;  /* TRUE or FALSE */
-l_int32             nbytes85;
-size_t              nbytescomp;
-L_COMPRESSED_DATA  *cid;
-FILE               *fp;
-
-    PROCNAME("l_generateG4Data");
-
-    if (!fname)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("fname not defined",
-                                              procName, NULL);
-
-        /* The returned ccitt g4 data in memory is the block of
-         * bytes in the tiff file, starting after 8 bytes and
-         * ending before the directory. */
-    if (extractG4DataFromFile(fname, &datacomp, &nbytescomp,
-                              &w, &h, &minisblack)) {
-        return (L_COMPRESSED_DATA *)ERROR_PTR("datacomp not extracted",
-                                              procName, NULL);
-    }
-
-        /* Read the resolution */
-    if ((fp = fopenReadStream(fname)) == NULL)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("stream not opened",
-                                              procName, NULL);
-    getTiffResolution(fp, &xres, &yres);
-    fclose(fp);
-
-        /* Optionally, encode the compressed data */
-    if (ascii85flag == 1) {
-        data85 = encodeAscii85(datacomp, nbytescomp, &nbytes85);
-        FREE(datacomp);
-        if (!data85)
-            return (L_COMPRESSED_DATA *)ERROR_PTR("data85 not made",
-                                                  procName, NULL);
-        else
-            data85[nbytes85 - 1] = '\0';  /* remove the newline */
-    }
-
-    cid = (L_COMPRESSED_DATA *)CALLOC(1, sizeof(L_COMPRESSED_DATA));
-    if (!cid)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("cid not made", procName, NULL);
-    if (ascii85flag == 0)
-        cid->datacomp = datacomp;
-    else {  /* ascii85 */
-        cid->data85 = data85;
-        cid->nbytes85 = nbytes85;
-    }
-    cid->type = L_G4_ENCODE;
-    cid->nbytescomp = nbytescomp;
-    cid->w = w;
-    cid->h = h;
-    cid->minisblack = minisblack;
-    cid->res = xres;
-    return cid;
-}
-
-
 /*-------------------------------------------------------------*
  *                     For tiff multipage files                *
  *-------------------------------------------------------------*/
@@ -1778,10 +1498,10 @@ l_int32
 convertFlateToPSEmbed(const char  *filein,
                       const char  *fileout)
 {
-char               *outstr;
-l_int32             w, h, nbytes;
-l_float32           xpt, ypt, wpt, hpt;
-L_COMPRESSED_DATA  *cid;
+char         *outstr;
+l_int32       w, h, nbytes;
+l_float32     xpt, ypt, wpt, hpt;
+L_COMP_DATA  *cid;
 
     PROCNAME("convertFlateToPSEmbed");
 
@@ -1802,8 +1522,7 @@ L_COMPRESSED_DATA  *cid;
     if (w * 11.0 > h * 8.5) {
         wpt = 572.0;   /* 612 - 2 * 20 */
         hpt = wpt * (l_float32)h / (l_float32)w;
-    }
-    else {
+    } else {
         hpt = 752.0;   /* 792 - 2 * 20 */
         wpt = hpt * (l_float32)w / (l_float32)h;
     }
@@ -1818,7 +1537,7 @@ L_COMPRESSED_DATA  *cid;
     if (l_binaryWrite(fileout, "w", outstr, nbytes))
         return ERROR_INT("ps string not written to file", procName, 1);
     FREE(outstr);
-    compressed_dataDestroy(&cid);
+    l_CIDataDestroy(&cid);
     return 0;
 }
 
@@ -1962,9 +1681,9 @@ convertFlateToPSString(const char  *filein,
                        l_int32      pageno,
                        l_int32      endpage)
 {
-char               *outstr;
-l_float32           xpt, ypt, wpt, hpt;
-L_COMPRESSED_DATA  *cid;
+char         *outstr;
+l_float32     xpt, ypt, wpt, hpt;
+L_COMP_DATA  *cid;
 
     PROCNAME("convertFlateToPSString");
 
@@ -2002,8 +1721,9 @@ L_COMPRESSED_DATA  *cid;
 #if  DEBUG_FLATE
     fprintf(stderr, "w = %d, h = %d, bps = %d, spp = %d\n",
             cid->w, cid->h, cid->bps, cid->spp);
-    fprintf(stderr, "uncomp bytes = %d, comp bytes = %d, nbytes85 = %d\n",
-            cid->nbytes, cid->nbytescomp, cid->nbytes85);
+    fprintf(stderr, "uncomp bytes = %ld, comp bytes = %ld, nbytes85 = %ld\n",
+            (unsigned long)cid->nbytes, (unsigned long)cid->nbytescomp,
+            (unsigned long)cid->nbytes85);
     fprintf(stderr, "xpt = %7.2f, ypt = %7.2f, wpt = %7.2f, hpt = %7.2f\n",
              xpt, ypt, wpt, hpt);
 #endif   /* DEBUG_FLATE */
@@ -2014,7 +1734,7 @@ L_COMPRESSED_DATA  *cid;
         return ERROR_INT("outstr not made", procName, 1);
     *poutstr = outstr;
     *pnbytes = strlen(outstr);
-    compressed_dataDestroy(&cid);
+    l_CIDataDestroy(&cid);
     return 0;
 }
 
@@ -2035,14 +1755,14 @@ L_COMPRESSED_DATA  *cid;
  *      Return: PS string, or null on error
  */
 char *
-generateFlatePS(const char         *filein,
-                L_COMPRESSED_DATA  *cid,
-                l_float32           xpt,
-                l_float32           ypt,
-                l_float32           wpt,
-                l_float32           hpt,
-                l_int32             pageno,
-                l_int32             endpage)
+generateFlatePS(const char   *filein,
+                L_COMP_DATA  *cid,
+                l_float32     xpt,
+                l_float32     ypt,
+                l_float32     wpt,
+                l_float32     hpt,
+                l_int32       pageno,
+                l_int32       endpage)
 {
 l_int32  w, h, bps, spp;
 char    *outstr;
@@ -2099,11 +1819,11 @@ SARRAY  *sa;
         sarrayAddString(sa, (char *)"  <~", L_COPY);
         sarrayAddString(sa, cid->cmapdata85, L_INSERT);
         sarrayAddString(sa, (char *)"  ] setcolorspace", L_COPY);
-    }
-    else if (spp == 1)
+    } else if (spp == 1) {
         sarrayAddString(sa, (char *)"/DeviceGray setcolorspace", L_COPY);
-    else   /* spp == 3 */
+    } else {  /* spp == 3 */
         sarrayAddString(sa, (char *)"/DeviceRGB setcolorspace", L_COPY);
+    }
 
     sarrayAddString(sa,
               (char *)"/RawData currentfile /ASCII85Decode filter def", L_COPY);
@@ -2120,16 +1840,16 @@ SARRAY  *sa;
     sprintf(bigbuf, "     /ImageMatrix [ %d 0 0 %d 0 %d ]", w, -h, h);
     sarrayAddString(sa, bigbuf, L_COPY);
 
-    if (cid->cmapdata85)
+    if (cid->cmapdata85) {
         sarrayAddString(sa, (char *)"     /Decode [0 255]", L_COPY);
-    else if (spp == 1) {
+    } else if (spp == 1) {
         if (bps == 1)  /* miniswhite photometry */
             sarrayAddString(sa, (char *)"     /Decode [1 0]", L_COPY);
         else  /* bps > 1 */
             sarrayAddString(sa, (char *)"     /Decode [0 1]", L_COPY);
-    }
-    else  /* spp == 3 */
+    } else {  /* spp == 3 */
         sarrayAddString(sa, (char *)"     /Decode [0 1 0 1 0 1]", L_COPY);
+    }
 
     sarrayAddString(sa, (char *)"     /DataSource Data", L_COPY);
     sarrayAddString(sa, (char *)"  >> image", L_COPY);
@@ -2152,158 +1872,6 @@ SARRAY  *sa;
 }
 
 
-/*!
- *  l_generateFlateData()
- *
- *      Input:  fname
- *              ascii85flag (0 for gzipped; 1 for ascii85-encoded gzipped)
- *      Return: cid (flate compressed image data), or null on error
- *
- *  Notes:
- *      (1) The input image is converted to one of these 4 types:
- *           - 1 bpp
- *           - 8 bpp, no colormap
- *           - 8 bpp, colormap
- *           - 32 bpp rgb
- *      (2) Set ascii85flag:
- *           - 0 for binary data (not permitted in PostScript)
- *           - 1 for ascii85 (5 for 4) encoded binary data
- */
-L_COMPRESSED_DATA *
-l_generateFlateData(const char  *fname,
-                    l_int32      ascii85flag)
-{
-L_COMPRESSED_DATA  *cid;
-PIX                *pixs;
-
-    PROCNAME("l_generateFlateData");
-
-    if (!fname)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("fname not defined",
-                                              procName, NULL);
-
-    if ((pixs = pixRead(fname)) == NULL)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("pixs not made",
-                                              procName, NULL);
-    cid = pixGenerateFlateData(pixs, ascii85flag);
-    pixDestroy(&pixs);
-    return cid;
-}
-
-
-/*!
- *  pixGenerateFlateData()
- *
- *      Input:  pixs
- *              ascii85flag (0 for gzipped; 1 for ascii85-encoded gzipped)
- *      Return: cid (flate compressed image data), or null on error
- */
-L_COMPRESSED_DATA *
-pixGenerateFlateData(PIX     *pixs,
-                     l_int32  ascii85flag)
-{
-l_uint8       *data = NULL;  /* uncompressed raster data in required format */
-l_uint8       *datacomp = NULL;  /* gzipped raster data */
-char          *data85 = NULL;  /* ascii85 encoded gzipped raster data */
-l_uint8       *cmapdata = NULL;  /* uncompressed colormap */
-char          *cmapdata85 = NULL;  /* ascii85 encoded uncompressed colormap */
-char          *cmapdatahex = NULL;  /* hex ascii uncompressed colormap */
-l_int32        ncolors;  /* in colormap; not used if cmapdata85 is null */
-l_int32        bps;  /* bits/sample: usually 8 */
-l_int32        spp;  /* samples/pixel: 1-grayscale); 3-rgb; 4-rgba */
-l_int32        w, h, d, cmapflag;
-l_int32        ncmapbytes, ncmapbytes85, nbytes85;
-size_t         nbytes, nbytescomp;
-L_COMPRESSED_DATA  *cid;
-PIX           *pixt;
-PIXCMAP       *cmap;
-
-    PROCNAME("pixGenerateFlateData");
-
-    if (!pixs)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("pixs not defined",
-                                              procName, NULL);
-
-        /* Convert the image to one of these 4 types:
-         *     1 bpp
-         *     8 bpp, no colormap
-         *     8 bpp, colormap
-         *     32 bpp rgb    */
-    pixGetDimensions(pixs, &w, &h, &d);
-    cmap = pixGetColormap(pixs);
-    cmapflag = (cmap) ? 1 : 0;
-    if (d == 2 || d == 4 || d == 16) {
-        pixt = pixConvertTo8(pixs, cmapflag);
-        cmap = pixGetColormap(pixt);
-        d = pixGetDepth(pixt);
-    }
-    else
-        pixt = pixClone(pixs);
-    spp = (d == 32) ? 3 : 1;
-    bps = (d == 32) ? 8 : d;
-
-        /* Extract and encode the colormap data as both ascii85 and hexascii  */
-    ncolors = 0;
-    if (cmap) {
-        pixcmapSerializeToMemory(cmap, 3, &ncolors, &cmapdata, &ncmapbytes);
-        if (!cmapdata)
-            return (L_COMPRESSED_DATA *)ERROR_PTR("cmapdata not made",
-                                                  procName, NULL);
-
-        cmapdata85 = encodeAscii85(cmapdata, ncmapbytes, &ncmapbytes85);
-        cmapdatahex = pixcmapConvertToHex(cmapdata, ncmapbytes, ncolors);
-        FREE(cmapdata);
-    }
-
-        /* Extract and compress the raster data */
-    pixGetRasterData(pixt, &data, &nbytes);
-    pixDestroy(&pixt);
-    datacomp = zlibCompress(data, nbytes, &nbytescomp);
-    if (!datacomp) {
-        if (cmapdata85) FREE(cmapdata85);
-        if (cmapdatahex) FREE(cmapdatahex);
-        return (L_COMPRESSED_DATA *)ERROR_PTR("datacomp not made",
-                                              procName, NULL);
-    }
-    FREE(data);
-
-        /* Optionally, encode the compressed data */
-    if (ascii85flag == 1) {
-        data85 = encodeAscii85(datacomp, nbytescomp, &nbytes85);
-        FREE(datacomp);
-        if (!data85) {
-            FREE(cmapdata85);
-            return (L_COMPRESSED_DATA *)ERROR_PTR("data85 not made",
-                                                  procName, NULL);
-        }
-        else
-            data85[nbytes85 - 1] = '\0';  /* remove the newline */
-    }
-
-    cid = (L_COMPRESSED_DATA *)CALLOC(1, sizeof(L_COMPRESSED_DATA));
-    if (!cid)
-        return (L_COMPRESSED_DATA *)ERROR_PTR("cid not made", procName, NULL);
-    if (ascii85flag == 0)
-        cid->datacomp = datacomp;
-    else {  /* ascii85 */
-        cid->data85 = data85;
-        cid->nbytes85 = nbytes85;
-    }
-    cid->type = L_FLATE_ENCODE;
-    cid->cmapdatahex = cmapdatahex;
-    cid->cmapdata85 = cmapdata85;
-    cid->nbytescomp = nbytescomp;
-    cid->ncolors = ncolors;
-    cid->w = w;
-    cid->h = h;
-    cid->bps = bps;
-    cid->spp = spp;
-    cid->res = pixGetXRes(pixs);
-    cid->nbytes = nbytes;  /* only for debugging */
-    return cid;
-}
-
-
 /*---------------------------------------------------------------------*
  *                          Write to memory                            *
  *---------------------------------------------------------------------*/
@@ -2517,8 +2085,7 @@ l_int32   eof, index, nread, nbout, i;
     if (inword == 0) {
         outbuf[0] = 'z';
         nbout = 1;
-    }
-    else { /* output nread + 1 bytes */
+    } else { /* output nread + 1 bytes */
         for (i = 4; i >= 4 - nread; i--) {
             val = inword / power85[i];
             outbuf[4 - i] = (l_uint8)(val + '!');
@@ -2581,9 +2148,9 @@ l_uint32  oword;
         val = inc - '!';
         if (val < 85) {
             oword = oword * 85 + val;
-            if (bytecount < 4)
+            if (bytecount < 4) {
                 bytecount++;
-            else {  /* we have all 5 input chars for the oword */
+            } else {  /* we have all 5 input chars for the oword */
                 outa[ocount] = (oword >> 24) & 0xff;
                 outa[ocount + 1] = (oword >> 16) & 0xff;
                 outa[ocount + 2] = (oword >> 8) & 0xff;
@@ -2592,15 +2159,13 @@ l_uint32  oword;
                 bytecount = 0;
                 oword = 0;
             }
-        }
-        else if (inc == 'z' && bytecount == 0) {
+        } else if (inc == 'z' && bytecount == 0) {
             outa[ocount] = 0;
             outa[ocount + 1] = 0;
             outa[ocount + 2] = 0;
             outa[ocount + 3] = 0;
             ocount += 4;
-        }
-        else if (inc == '~') {  /* end of data */
+        } else if (inc == '~') {  /* end of data */
             fprintf(stderr, " %d extra bytes output\n", bytecount - 1);
             switch (bytecount) {
             case 0:   /* normal eof */
